
// Adafruit Motor Shield Commander, an easy way to control motors output via serial commands
// Developed by Camille Moussette during an internship at Microsoft Research Redmond, Spring 2011
// freqout code by Paul Badger 

// requires Adafruit Motor Shield, http://www.adafruit.com/products/81
// works with Messenger library version 1.5, http://arduino.cc/playground/Code/Messenger

// commands need the following structure 
// Channel Direction Power Duration
// id[1-4,5-6,9] direction[-1,0,1] speed[0,255] duration[0,10000]
// "1 1 255 50" will power Motor1 forward at full speed for 50 ms
// "5 1 180 0" will move servo1 at 180 degrees, direction and duration are not relevant in this case
// "9 1 105 15" will power produce a train of square pulses on Motor1 @ 105 Hz for 15 ms, direction is not relevant
// 1-4:generic motor or 5-6:servo, 9:freq burst on motor1


#include <Messenger.h>
#include <AFMotor.h>
#include <Servo.h>   // works with a servo motor, not DC http://arduino.cc/en/reference/servo
#include <math.h>  

// Instantiate Messenger object with the default separator (the space character)
Messenger message = Messenger(); 

// AF motor
AF_DCMotor motors[4] = {
  AF_DCMotor(1),
  AF_DCMotor(2),
  AF_DCMotor(3),
  AF_DCMotor(4)
  }; 
  // servo
  Servo servo1, servo2;

// variables
int incomingData[10];
int commandData[4];
boolean newCommand, doneCommand = false;
long commandEndsAt[6];

// controls
/*int knobAPin = A0;
 int knobBPin = A1;
 int knobA = 0;
 int knobB = 0;
 */
int serialData;
int servoTest = 0;

void setup(void)
{
  // init serial
  Serial.begin(115200);

  // Attach the callback function to the Messenger
  message.attach(messageReady);

  // init motors
  for (int i = 0; i< 4; i++){
    motors[i].setSpeed(0);
    motors[i].run(RELEASE);
  }
  servo1.attach(9);
  servo1.write(95);
  servo2.attach(10);
  servo2.write(95);
  
  Serial.println("Arduino ready!");
  Serial.println("Expecting commands following the structure: 'Channel Direction Power Duration'");
  Serial.println("'1 1 255 50' will power Motor1 forward at full speed for 50 ms");

}

void loop(void)
{

  // The following line is the most effective way of using Serial and Messenger's callback
  while ( Serial.available() )  message.process(Serial.read () );

  
  
  // scan motors to stop expired commands
  for(int k = 0; k <6; k++){
    if(commandEndsAt[k] > 0){
      // check if the commands is expired
      if(millis() > commandEndsAt[k]){
        
        if(k < 4){
          // stop motor

          motors[k].run(RELEASE);
        }
        else if(k == 4){
          // reset servo
          //servo1.detach();
          servo1.write(95); 
        }
        else if(k == 5){
          // reset servo
          //servo2.detach();
          servo2.write(95); 
        }
        //reset
        Serial.println("reset command");
        commandEndsAt[k] = 0;
      }
    }
  }

  // check new command
  if(newCommand){
    // print new command
    Serial.print("received command ");
    for(int j = 0; j < 4; j++){
      Serial.print(commandData[j], DEC);
      Serial.print(";");
    }
    Serial.println(" ");

    // parse command
    int motorId = commandData[0] - 1;
    int motorDir = commandData[1];
    int motorSpeed = commandData[2];
    int motorDuration = commandData[3];

    // set end timer
    commandEndsAt[motorId] = millis() + motorDuration;

    // check what type of motor we control, 0-3:generic or 4-5:servo, 8:freq burst, remember -1 from input string
    if(motorId < 4){
      // set speed
      motors[motorId].setSpeed(motorSpeed);


      Serial.println(motorSpeed);
      if(motorDir == 1){
        motors[motorId].run(FORWARD);
        Serial.print("M");
        Serial.print(motorId+1);
        Serial.print(" running forward for ");
        Serial.print(motorDuration);
        Serial.println("ms");
      }
      else if(motorDir == -1){
        motors[motorId].run(BACKWARD);
        Serial.print("M");
        Serial.print(motorId+1);
        Serial.print(" running backward for ");
        Serial.print(motorDuration);
        Serial.println("ms");
      }
      else{
        motors[motorId].run(RELEASE);
      }

    }
    else if(motorId == 4){
      Serial.print("moving servo1 to ");
      Serial.println(motorSpeed);
      // run motor
      //servo1.attach(9);
      servo1.write(motorSpeed);
    }
    else if(motorId == 5){
      Serial.print("moving servo1 to ");
      Serial.println(motorSpeed);
      // run motor
      //servo2.attach(10);
      servo1.write(motorSpeed);
    }
    
    else if(motorId == 8){
      Serial.print("freq burst mode: ");
      Serial.print(motorSpeed);
      Serial.print("hz ");
      Serial.print(motorDuration);
      Serial.println("ms ");
      // run freq sequence
      freqout(motorSpeed, motorDuration, 0);  // freqout(freq, duration, motorId:0-3)

    }

    newCommand = false;
  }

}

// Create the callback function
void messageReady() {

  int pos = 0;
  // Loop through all the available elements of the message
  while ( message.available() ) {
    // strtucture of the message, all ints separated by whitespace
    // id[1-4] direction[-1,0,1] speed[0,255] duration[0,10000]
    incomingData[pos] = message.readInt();
    //Serial.print(incomingData[0]);
    pos++;
  }

  // if we have a good size packet
  if(pos == 4){
    for(int j = 0; j < 4; j++){
      commandData[j] = incomingData[j];
      newCommand = true;

    }
  }
  else{
    // clear array
    for(int j = 0; j < 10; j++){
      incomingData[j] = 0;
    }
  }

}

void freqout(int freq, int t, int id)  // freq in hz, t in ms
{
  int hperiod;                               //calculate 1/2 period in us
  long cycles, i;

  hperiod = (500000 / freq) - 7;             // subtract 7 us to make up for digitalWrite overhead

  cycles = ((long)freq * (long)t) / 1000;    // calculate cycles
  Serial.print(freq);
 // Serial.print((char)9);                   // ascii 9 is tab - you have to coerce it to a char to work 
 // Serial.print(hperiod);
 // Serial.print((char)9);
 // Serial.println(cycles);

  for (i=0; i<= cycles; i++){              // play note for t ms 
    motors[id].setSpeed(255);
    motors[id].run(FORWARD); 
    delayMicroseconds(hperiod);
    motors[id].run(RELEASE); 
    delayMicroseconds(hperiod - 1);     // - 1 to make up for digitaWrite overhead
  }

}

